# coding: utf-8
import six
from django.db import models
from django.conf import settings
from swutils.encrypt import decrypt, encrypt


class OneValueModel(models.Model):
    """
        Base model for key-value data storing.
        Example - system statues, counters and other.
    """
    NAME_CHOICES = None     # Available keys. Must be defined at sub class

    name = models.CharField(max_length=100, unique=True, choices=NAME_CHOICES)
    value = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        abstract = True

    @classmethod
    def get_value(cls, name, default=None, decrypt_it=False):
        instance, created = cls.objects.get_or_create(name=name, defaults={'value': default})
        value = instance.value

        if decrypt_it:
            value = decrypt(value, key=cls._get_key())

        return value

    @classmethod
    def set_value(cls, name, value, encrypt_it=False):
        if encrypt_it:
            value = encrypt(value, key=cls._get_key())

        cls.objects.get_or_create(name=name)    # create if needed
        return cls.objects.filter(name=name).update(value=value)

    @classmethod
    def _get_key(cls):
        key = settings.SECRET_KEY
        if isinstance(key, six.text_type):
            key = key.encode()
        return key


# coding: utf-8
from io import StringIO
from django.core.management.commands.dumpdata import Command as Dumpdata


class Command(Dumpdata):
    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)

    def handle(self, *args, **kwargs):
        orig_stdout = self.stdout
        self.stdout = StringIO()
        super(Command, self).handle(*args, **kwargs)
        data = self.stdout.getvalue()

        data = data.encode("utf-8").decode("unicode_escape")

        orig_stdout.write(data)

        return data


# coding: utf-8
import json
from collections import OrderedDict

from django.forms import model_to_dict
from django.core.serializers.json import DjangoJSONEncoder


def object_to_json(obj, indent=2):
    """
        transform object to json
    """
    instance_json = json.dumps(obj, indent=indent, ensure_ascii=False, cls=DjangoJSONEncoder)
    return instance_json


def model_to_json(model_instance):
    """
        transform instance to json
    """
    instance_dict = model_to_dict(model_instance)
    return object_to_json(instance_dict)


def qs_to_json(qs, fields=None):
    """
    transform QuerySet to json
    """
    if not fields :
        fields = [f.name for f in qs.model._meta.fields]


    # сформируем список для сериализации
    objects = []
    for value_dict in qs.values(*fields):
        # сохраним порядок полей, как определено в моделе
        o = OrderedDict()
        for f in fields:
            o[f] = value_dict[f]
        objects.append(o)

    # сериализуем
    json_qs = json.dumps(objects, indent=2, ensure_ascii=False, cls=DjangoJSONEncoder)
    return json_qs


def mongoqs_to_json(qs, fields=None):
    """
    transform mongoengine.QuerySet to json
    """

    l = list(qs.as_pymongo())

    for element in l:
        element.pop('_cls')

    # use DjangoJSONEncoder for transform date data type to datetime
    json_qs = json.dumps(l, indent=2, ensure_ascii=False, cls=DjangoJSONEncoder)
    return json_qs





# coding: utf-8
import datetime
from djutils.date_utils import iso_to_datetime, iso_to_date, date_to_datetime, date_to_datetime_lte


def add_date_from_to_arguments(parser):
    parser.add_argument('--from',
                        dest='from',
                        help=u'начало выборки с дата/время изменения данных (например, 2011-07-22 12:45:00)', )
    parser.add_argument('--to',
                        dest='to',
                        help=u'конец выборки по дата/время изменения данных (например, 2011-07-22 20:00:00)', )
    parser.add_argument('--last_week',
                        dest='last_week',
                        help=u'обновить данные за последнюю неделю',
                        default=False,
                        action="store_true", )
    parser.add_argument('--last_day',
                        dest='last_day',
                        help=u'обновить данные за последний день',
                        default=False,
                        action="store_true", )
    parser.add_argument('--last_2hours',
                        dest='last_2hours',
                        help=u'обновить данные за последние 2 часа',
                        default=False,
                        action="store_true", )


def process_date_from_to_options(options, to_datetime=False, default_dt_to=False):
    """
        to_datetime - приводить ли date к datetime
        default_dt_to - устанавливать заведомо будущее дефолтное значение для dt_to
    """
    start_time = datetime.datetime.now()

    if options.get('last_week'):
        dt_from = start_time - datetime.timedelta(days=7)
        dt_to = start_time

    elif options.get('last_day'):
        dt_from = start_time - datetime.timedelta(days=1)
        dt_to = start_time

    elif options.get('last_2hours'):
        dt_from = start_time - datetime.timedelta(hours=2)
        dt_to = start_time

    else:
        from_str = options.get('from')
        if from_str:
            try:
                dt_from = iso_to_datetime(from_str)
            except:
                dt_from = iso_to_date(from_str)
        else:
            dt_from = None

        to_str = options.get('to')
        if to_str:
            try:
                dt_to = iso_to_datetime(to_str)
            except:
                dt_to = iso_to_date(to_str)
        else:
            dt_to = None

    if default_dt_to and not dt_to:
        dt_to = datetime.datetime(2100, 1, 1)

    if to_datetime:
        if isinstance(dt_from, datetime.date):
            dt_from = date_to_datetime(dt_from)
        if isinstance(dt_to, datetime.date):
            dt_to = date_to_datetime_lte(dt_to)

    return dt_from, dt_to



# coding: utf-8
from django.http import HttpResponse
from . import json_utils

class JSONResponse(HttpResponse):
    """
    An HttpResponse that renders its content into JSON.
    """
    def __init__(self, data, **kwargs):
        content = json_utils.object_to_json(data)
        kwargs['content_type'] = 'application/json; charset=utf-8'
        super(JSONResponse, self).__init__(content, **kwargs)


# coding: utf-8
import calendar
from random import randrange
import datetime


def date_to_datetime(date):
    if date:
        return datetime.datetime(*(date.timetuple()[:6]))
    else:
        return None


def date_to_datetime_lte(date):
    return datetime.datetime.combine(date, datetime.time(23, 59, 59))


def iso_to_date(date_iso):
    if date_iso:
        return datetime.datetime.strptime(date_iso[:10], '%Y-%m-%d').date()
    else:
        return None


def iso_to_datetime(datetime_iso):
    if datetime_iso:
        return datetime.datetime.strptime(datetime_iso, '%Y-%m-%d %H:%M:%S')
    else:
        return None


def rus_to_date(date_rus):
    if date_rus:
        return datetime.datetime.strptime(date_rus, '%d.%m.%Y').date()
    else:
        return None


def date_to_timestamp(date):
    """
        date to unix timestamp in milliseconds
    """
    date_tuple = date.timetuple()
    timestamp = calendar.timegm(date_tuple) * 1000
    return timestamp


def random_date(dt_from, dt_to):
    """
    This function will return a random datetime between two datetime objects.
    :param start:
    :param end:
    """
    delta = dt_to - dt_from
    int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
    random_second = randrange(int_delta)
    return dt_from + datetime.timedelta(seconds=random_second)


def convert_timedelta_to_datetime(duration):
    return datetime.datetime.min + duration


# coding: utf-8
from django.http import HttpResponse, JsonResponse
from django.template import Template, RequestContext
from django.contrib.auth import login as auth_login, logout as auth_logout
from django.contrib.auth.forms import AuthenticationForm
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST, require_GET
from djutils.forms import transform_form_error


@csrf_exempt
@require_POST
def login(request):
    form = AuthenticationForm(request, data=request.POST)
    if form.is_valid():
        auth_login(request, form.get_user())
        return JsonResponse({'login': 'ok'})
    else:
        errors = transform_form_error(form)
        c = {'errors': errors}
        return JsonResponse(c, status=403)

@csrf_exempt
@require_POST
def logout(request):
    auth_logout(request)
    return JsonResponse({'logout': 'ok'})


@require_GET
def get_current_user(request):
    c = {
        'is_authenticated': request.user.is_authenticated(),
    }
    if request.user.is_authenticated():
        c.update({
            'username': request.user.username,
            'first_name': request.user.first_name,
            'last_name': request.user.last_name,
            'email': request.user.email,
            'is_staff': request.user.is_staff,
            'is_active': request.user.is_active,
        })
    return JsonResponse(c)



# coding: utf-8
import time
import socket
from django.conf import settings


def push(metric, value, timestamp=None, server=None, port=None):
    if not timestamp:
        timestamp = int(time.time())
    if not server:
        server = settings.GRAPHITE_HOST
    if not port:
        port = settings.GRAPHITE_PORT

    sock = socket.socket()
    sock.connect((server, int(port)))
    message = '%s %s %d\n' % (metric, value, timestamp)
    sock.sendall(message)
    sock.close()



# coding: utf-8
import re

from django.utils.http import urlquote
from django.utils.safestring import mark_safe

def url_path(request, base_url=None, is_full=False, *args, **kwargs):
    """
    join base_url and some GET-parameters to one; it could be absolute url optionally

    usage example:

        c['current_url'] = url_path(request, use_urllib=True, is_full=False)
        ...
        <a href="{{ current_url }}">Лабораторный номер</a>

    """
    if not base_url:
        base_url = request.path
        if is_full:
            protocol = 'https' if request.is_secure() else 'http'
            base_url = '%s://%s%s' % (protocol, request.get_host(), base_url)

    params = url_params(request, *args, **kwargs)
    url = '%s%s' % (base_url, params)
    return url

def url_params(request, except_params=None, as_is=False):
    """
    create string with GET-params of request

    usage example:
        c['sort_url'] = url_params(request, except_params=('sort',))
        ...
        <a href="{{ sort_url }}&sort=lab_number">Лабораторный номер</a>
    """
    if not request.GET:
        return ''
    params = []
    for key, value in request.GET.items():
        if except_params and key not in except_params:
            for v in request.GET.getlist(key):
                params.append('%s=%s' % (key, urlquote(v)))

    if as_is:
        str_params = '?' + '&'.join(params)
    else:
        str_params = '?' + '&'.join(params)
        str_params = urlquote(str_params)
    return mark_safe(str_params)


def prepare_sort_params(params, request, sort_key='sort', revers_sort=None, except_params=None):
    """
        Prepare sort params. Add revers '-' if need.
        Params:
            params - list of sort parameters
            request
            sort_key
            revers_sort - list or set with keys that need reverse default sort direction
            except_params - GET-params that will be ignored
        Example:
            view:
                c['sort_params'] = prepare_sort_params(
                    ('order__lab_number', 'order__client__lname', 'organization', 'city', 'street', ),
                    request,
                )
            template:
                   <th><a href="{{ sort_params.order__lab_number.url }}">Лабораторный номер</a></th>
               or
                    {% load djutils %}
                    ...
                    {% sort_th 'order__lab_number' 'Лабораторный номер' %}


    """
    current_param, current_reversed = sort_key_process(request, sort_key)

    except_params = except_params or []
    except_params.append(sort_key)

    base_url = url_params(request, except_params=except_params, as_is=True)

    sort_params = {}
    revers_sort = revers_sort or set()
    url_connector = '?' if request.get_full_path() == request.path else "&"
    for p in params:
        sort_params[p] = {}
        if current_param and p == current_param:
            prefix = '' if current_reversed else '-'
            sort_params[p]['url'] = base_url + "%s%s=%s" % (url_connector, sort_key, prefix + current_param)
            sort_params[p]['is_reversed'] = current_reversed
            sort_params[p]['is_current'] = True
        else:
            default_direction = '-' if p in revers_sort else ''
            sort_params[p]['url'] = base_url + "%s%s=%s%s" % (url_connector, sort_key, default_direction, p)
            sort_params[p]['is_reversed'] = False
            sort_params[p]['is_current'] = False

    return sort_params


def sort_key_process(request, sort_key='sort'):
    """
        process sort-parameter value (for example, "-name")
        return:
            current_param - field for sorting ("name)
            current_reversed - revers flag (True)
    """
    current = request.GET.get(sort_key)
    current_reversed = False
    current_param = None
    if current:
        mo = re.match(r'^(-?)(\w+)$', current)    # exclude first "-" (if exist)
        if mo:
            current_reversed = mo.group(1) == '-'
            current_param = mo.group(2)

    return current_param, current_reversed



# coding: utf-8
import re
from urllib.parse import quote
from django import http
from django.conf import settings
from django.http import HttpResponseRedirect

XS_SHARING_ALLOWED_ORIGINS = getattr(settings, 'XS_SHARING_ALLOWED_ORIGINS', '*')
XS_SHARING_ALLOWED_METHODS = getattr(settings, 'XS_SHARING_ALLOWED_METHODS', ['POST', 'GET', 'OPTIONS', 'PUT', 'DELETE'])
XS_SHARING_ALLOWED_HEADERS = getattr(settings, 'XS_SHARING_ALLOWED_HEADERS', ['Content-Type', '*'])
XS_SHARING_ALLOWED_CREDENTIALS = getattr(settings, 'XS_SHARING_ALLOWED_CREDENTIALS', 'true')


class XsSharing(object):
    """
    This middleware allows cross-domain XHR using the html5 postMessage API.

    Access-Control-Allow-Origin: http://foo.example
    Access-Control-Allow-Methods: POST, GET, OPTIONS, PUT, DELETE

    Based off https://gist.github.com/426829
    """
    def process_request(self, request):
        if 'HTTP_ACCESS_CONTROL_REQUEST_METHOD' in request.META:
            response = http.HttpResponse()
            if settings.DEBUG and 'HTTP_ORIGIN' in request.META:
                response['Access-Control-Allow-Origin'] = request.META['HTTP_ORIGIN']
            else:
                response['Access-Control-Allow-Origin'] = XS_SHARING_ALLOWED_ORIGINS
            response['Access-Control-Allow-Methods'] = ",".join( XS_SHARING_ALLOWED_METHODS )
            response['Access-Control-Allow-Headers'] = ",".join( XS_SHARING_ALLOWED_HEADERS )
            response['Access-Control-Allow-Credentials'] = XS_SHARING_ALLOWED_CREDENTIALS
            return response

        return None

    def process_response(self, request, response):
        if settings.DEBUG and 'HTTP_ORIGIN' in request.META:
            response['Access-Control-Allow-Origin'] = request.META['HTTP_ORIGIN']
        else:
            response['Access-Control-Allow-Origin'] = XS_SHARING_ALLOWED_ORIGINS
        response['Access-Control-Allow-Methods'] = ",".join( XS_SHARING_ALLOWED_METHODS )
        response['Access-Control-Allow-Headers'] = ",".join( XS_SHARING_ALLOWED_HEADERS )
        response['Access-Control-Allow-Credentials'] = XS_SHARING_ALLOWED_CREDENTIALS

        return response


EXEMPT_URLS = [
    re.compile(getattr(settings, 'LOGIN_URL', '/login/')),
    re.compile(getattr(settings, 'LOGOUT_URL', '/logout/')),
]
if hasattr(settings, 'LOGIN_EXEMPT_URLS'):
    EXEMPT_URLS += [re.compile(expr) for expr in settings.LOGIN_EXEMPT_URLS]

class LoginRequired(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if not request.user.is_authenticated():
            path = request.path_info
            if not any(m.match(path) for m in EXEMPT_URLS):
                next = '?next=%s' % quote(path)
                url = settings.LOGIN_URL + next
                return HttpResponseRedirect(url)
        response = self.get_response(request)
        return response


from __future__ import print_function
from django.conf import settings
from django.test.runner import DiscoverRunner


class TestRunnerWithMongo(DiscoverRunner):
    mongodb_name = 'test_%s' % (settings.MONGODB['NAME'], )

    def get_addition_mongodb_aliases(self):

        return tuple()

    def _get_params(self):
        params = {
            'host': settings.MONGODB.get('HOST'),
        }
        if settings.MONGODB.get('PORT'):
            params['port'] = settings.MONGODB['PORT']
        if settings.MONGODB.get('USER'):
            params['username'] = settings.MONGODB['USER']
        if settings.MONGODB.get('PASSWORD'):
            params['password'] = settings.MONGODB['PASSWORD']
        return params

    def setup_databases(self, **kwargs):
        from mongoengine.connection import connect, disconnect, register_connection

        # main mongo database
        # disconnect()
        connect(self.mongodb_name, **self._get_params())
        print('Creating mongo test database %s for alias \'default\'' % self.mongodb_name)

        # additional databases
        addition_aliases = self.get_addition_mongodb_aliases()
        for alias in addition_aliases:
            print('Creating mongo test database %s for alias \'%s\'' % (self.mongodb_name, alias))
            disconnect(alias=alias)
            register_connection(alias, self.mongodb_name, **self._get_params())

        return super(TestRunnerWithMongo, self).setup_databases(**kwargs)

    def teardown_databases(self, old_config, **kwargs):
        from mongoengine.connection import connect, get_connection, disconnect, register_connection

        # main mongo database
        connection = get_connection()
        try:
            connection.drop_database(self.mongodb_name)
            print('Dropping mongo test database %s for alias \'default\'' % self.mongodb_name)
        except Exception as ex:
            print('Can not to drop mongo test database %s for alias \'default\': %s' % (self.mongodb_name, ex))
        # disconnect()

        # additional databases
        addition_aliases = self.get_addition_mongodb_aliases()
        for alias in addition_aliases:
            connection = get_connection(alias=alias)
            try:
                connection.drop_database(self.mongodb_name)
                print('Dropping mongo test database %s for alias \'%s\'' % (self.mongodb_name, alias))
            except Exception as ex:
                print('Can not to drop mongo test database %s for alias \'%s\': %s' % (self.mongodb_name, alias, ex))
            disconnect()

        super(TestRunnerWithMongo, self).teardown_databases(old_config, **kwargs)



class TearDownTestCaseMixin(object):

    def tearDownMongo(self):
        from mongoengine.connection import get_db

        db = get_db()
        collections = db.collection_names()
        for col in collections:
            if col == 'system.indexes':
                continue
            db[col].drop()




# coding: utf-8
import json

from django import template
import sqlparse
from djutils import date_utils

register = template.Library()


@register.filter('iso_to_date')
def iso_to_date(date_iso):
    return date_utils.iso_to_date(date_iso)


@register.filter('pretty_sql')
def pretty_sql(sql):
    try:
        sql = sqlparse.format(sql, reindent=True, keyword_case='upper')
        return sql
    except Exception:
        return sql


@register.filter('pretty_json')
def pretty_json(json_text):
    try:
        pretty_json_text = json.dumps(json_text, indent=4)
        return pretty_json_text
    except Exception:
        return json_text


@register.filter('get_value_from_dict')
def get_value_from_dict(dict_data, key):
    """
    usage example {{ your_dict|get_value_from_dict:your_key }}
    """
    if key:
        return dict_data.get(key)


@register.inclusion_tag('djutils/sort_th.html', takes_context=True)
def sort_th(context, sort_param_name, label):
    return {
        'is_current': context['sort_params'][sort_param_name]['is_current'],
        'is_reversed': context['sort_params'][sort_param_name]['is_reversed'],
        'url': context['sort_params'][sort_param_name]['url'],
        'label': label,
    }




# coding: utf-8
from django import forms
from django.utils.safestring import mark_safe


class NameInput(forms.TextInput):
    media = forms.Media(js=(['djutils/js/capitalize.js']))

    def render(self, name, *args, **kwargs):
        return super(NameInput, self).render(name, *args, **kwargs) + mark_safe(
            '''
                <script type="text/javascript">
                    capitalize('input[type=text][name={}]');
                </script>
            '''.format(name)
        )




# coding: utf-8
import six

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.views import redirect_to_login
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.shortcuts import redirect
from django.utils.encoding import force_text

from djutils.views.helpers import prepare_sort_params, url_path
from django.http import QueryDict
from django.views.generic.edit import FormMixin as DjangoFormMixin

# last django , django 1.8
try:
    from urllib.parse import quote
    from django.urls import reverse
    from django.contrib.auth.mixins import PermissionRequiredMixin as PermissionRequiredMixinAuth

except ImportError:
    from urllib import quote
    from django.core.urlresolvers import reverse

    class PermissionRequiredMixinAuth(object):
        login_url = None
        permission_denied_message = ''
        raise_exception = False
        permission_required = None
        redirect_field_name = REDIRECT_FIELD_NAME

        def get_login_url(self):
            login_url = self.login_url or settings.LOGIN_URL
            if not login_url:
                raise ImproperlyConfigured(
                    '{0} is missing the login_url attribute. Define {0}.login_url, settings.LOGIN_URL, or override '
                    '{0}.get_login_url().'.format(self.__class__.__name__)
                )
            return force_text(login_url)

        def get_permission_denied_message(self):
            return self.permission_denied_message

        def get_redirect_field_name(self):
            return self.redirect_field_name

        def handle_no_permission(self):
            if self.raise_exception:
                raise PermissionDenied(self.get_permission_denied_message())
            return redirect_to_login(self.request.get_full_path(), self.get_login_url(), self.get_redirect_field_name())

        def get_permission_required(self):
            if self.permission_required is None:
                raise ImproperlyConfigured(
                    '{0} is missing the permission_required attribute. Define {0}.permission_required, or override '
                    '{0}.get_permission_required().'.format(self.__class__.__name__)
                )
            if isinstance(self.permission_required, six.string_types):
                perms = (self.permission_required,)
            else:
                perms = self.permission_required
            return perms

        def has_permission(self):
            perms = self.get_permission_required()
            return self.request.user.has_perms(perms)

        def dispatch(self, request, *args, **kwargs):
            if not self.has_permission():
                return self.handle_no_permission()
            return super(PermissionRequiredMixin, self).dispatch(request, *args, **kwargs)


class TitleMixin(object):
    title = ''

    def get_title(self):
        return self.title

    def get_context_data(self, **kwargs):
        c = super(TitleMixin, self).get_context_data(**kwargs)

        title = self.get_title()
        if title:
            c['title'] = title

        return c


class LoggerMixin(object):
    logger = None

    def get(self, request, *args, **kwargs):
        response = super(LoggerMixin, self).get(request, *args, **kwargs)
        self.logger.debug(self.get_title(), extra={'GET': request.GET, 'username': request.user.username})
        return response

    def post(self, request, *args, **kwargs):
        title = self.get_title()
        response = super(LoggerMixin, self).post(request, *args, **kwargs)
        self.logger.info(title, extra={'POST': self.request.POST, 'username': self.request.user.username})
        return response


class SortMixin(object):
    sort_params = None          # must be defined
    sort_param_name = 'sort'
    sort_qs = True
    sort_default = True

    def get_queryset(self):
        qs = super(SortMixin, self).get_queryset()
        if self.sort_qs:
            order_by = self.request.GET[self.sort_param_name]
            qs = qs.order_by(order_by)
        return qs

    def get(self, request, *args, **kwargs):
        if not self.sort_default and not request.GET:
            return super().get(request, *args, **kwargs)

        if not request.GET.get(self.sort_param_name):
            redirect_url = request.get_full_path()
            if not request.GET:
                redirect_url += '?'
            else:
                redirect_url += '&'
            redirect_url += self.sort_param_name + '=' + self.get_default_sort_param()
            return redirect(redirect_url)

        response = super(SortMixin, self).get(request, *args, **kwargs)
        return response

    @classmethod
    def get_default_sort_param(cls):
        return cls.sort_params[0]

    def get_sort_except_params(self):
        except_params = []
        if hasattr(self, 'page_kwarg') and self.page_kwarg:
            except_params.append(self.page_kwarg)
        return except_params

    def get_context_data(self, **kwargs):
        c = super(SortMixin, self).get_context_data(**kwargs)
        c['sort_params'] = prepare_sort_params(
            self.sort_params,
            request=self.request,
            sort_key=self.sort_param_name,
            except_params=self.get_sort_except_params()
        )
        return c


class FilterMixin(object):
    filter = None

    def get_queryset(self):
        qs = super(FilterMixin, self).get_queryset()
        self.filter_obj = self.filter(self.request.GET, qs)
        return self.filter_obj.qs

    def get_context_data(self, **kwargs):
        c = super(FilterMixin, self).get_context_data(**kwargs)
        c['form'] = self.filter_obj.form
        return c


class PageMixin(object):
    def get_context_data(self, **kwargs):
        c = super(PageMixin, self).get_context_data(**kwargs)
        if c.get('page_obj'):
            c['start_counter'] = (c['page_obj'].number - 1) * c['page_obj'].paginator.per_page
        return c


class BreadcrumbsMixin(object):
    def get_breadcrumbs(self):
        return []

    def get_context_data(self, **kwargs):
        kwargs['breadcrumbs'] = self.get_breadcrumbs()
        return super(BreadcrumbsMixin, self).get_context_data(**kwargs)


class FormMixin(DjangoFormMixin):
    request_method = 'post'

    def form_process(self):
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def get(self, *args, **kwargs):
        if self.request_method == 'get':
            return self.form_process()

        return super(FormMixin, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        if self.request_method == 'post':
            return self.form_process()

        return super(FormMixin, self).post(*args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(FormMixin, self).get_form_kwargs()
        kwargs['data'] = getattr(self.request, self.request_method.upper(), None) or None
        return kwargs


class NextMixin(object):
    def get_next(self):
        return self.request.GET.get('next')

    def get_context_data(self, **kwargs):
        kwargs['next'] = self.get_next()
        return super(NextMixin, self).get_context_data(**kwargs)


class PermissionRequiredMixin(PermissionRequiredMixinAuth):
    def handle_no_permission(self):
        url = reverse('permission_denied')
        params = QueryDict(mutable=True)
        params['url'] = self.request.path

        previous_url = self.request.META.get('HTTP_REFERER')
        if previous_url:
            params['next'] = quote(previous_url)
        url += '?%s' % params.urlencode()

        return redirect(url)


class CurrentUrlMixin(object):
    is_full_current_url = False

    def get_context_data(self, **kwargs):
        c = super(CurrentUrlMixin, self).get_context_data(**kwargs)
        c['current_url'] = url_path(self.request, is_full=self.is_full_current_url)
        return c
